#include "game.h"

GAME::GAME(SDL_Surface * surface, FONT * font, SOUNDBANK * sounds)
{
	fprintf(stderr, "Load Life\n");
	life=IMG_Load(GFX_LIFE);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_LIFE);
		exit(-1);
	}
	fprintf(stderr, "Load Bucket\n");
	bucket=IMG_Load(GFX_BUCKET);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_BUCKET);
		exit(-1);
	}
	drip[0]=IMG_Load(GFX_DRIP);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_DRIP);
		exit(-1);
	}
	drip[1]=IMG_Load(GFX_GREEN_DRIP);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_GREEN_DRIP);
		exit(-1);
	}
	drip[2]=IMG_Load(GFX_RED_DRIP);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_RED_DRIP);
		exit(-1);
	}
	healthIcon=IMG_Load(GFX_HEALTH_ICON);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_HEALTH_ICON);
		exit(-1);
	}
	boostIcon=IMG_Load(GFX_BOOST_ICON);
	if(life==NULL)
	{
		fprintf(stderr, "Error: Unable to open file - %s\n", GFX_BOOST_ICON);
		exit(-1);
	}

	fprintf(stderr, "Set Transparents\n");
	fprintf(stderr, "Set Transparents for life\n");
	SDL_SetColorKey(life, SDL_SRCCOLORKEY, SDL_MapRGB(life->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for bucket\n");
	SDL_SetColorKey(bucket, SDL_SRCCOLORKEY, SDL_MapRGB(bucket->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for drip \n");
	SDL_SetColorKey(drip[0], SDL_SRCCOLORKEY, SDL_MapRGB(drip[0]->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for drip 1\n");
	SDL_SetColorKey(drip[1], SDL_SRCCOLORKEY, SDL_MapRGB(drip[1]->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for drip 2\n");
	SDL_SetColorKey(drip[2], SDL_SRCCOLORKEY, SDL_MapRGB(drip[2]->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for health\n");
	SDL_SetColorKey(healthIcon, SDL_SRCCOLORKEY, SDL_MapRGB(healthIcon->format, 255, 0, 255));
		fprintf(stderr, "Set Transparents for boost\n");
	SDL_SetColorKey(boostIcon, SDL_SRCCOLORKEY, SDL_MapRGB(boostIcon->format, 255, 0, 255));
		fprintf(stderr, "Set screen\n");
	screen=surface;
	scoreFont=font;
	/* load Sound Samples */
	soundBank=sounds;
	
	fprintf(stderr, "Load Sounds\n");
	snd_drop=soundBank->loadSound(SOUND_DRIP_2);
	snd_caught=soundBank->loadSound(SOUND_DRIP_3);
	snd_missed=soundBank->loadSound(SOUND_DRIP_4);
	snd_levelUp=soundBank->loadSound(SOUND_DRIP_5);
	snd_gameOver=soundBank->loadSound(SOUND_GAME_OVER);
	music=1;
	
	fprintf(stderr, "Reset\n");
	reset();
}
		
GAME::~GAME()
{
	SDL_FreeSurface(life);
}

void GAME::reset()
{
	score=0;
	oldScore=-1;
	lives=48;
	drips=NULL;
	dripCounter=0;
	dripRate=3000;
	threashold=7;
	speed=10;
	energy=4800;
}

void GAME::drawScore()
{
	char fmtScore[7];
	/* update score */
	sprintf(fmtScore, "%06d", score);
	scoreFont->drawTextOver(screen, 220, 0, 0, 0, 0, fmtScore);
}

int GAME::addScore(int value)
{
	score+=value;
	if((score%50)<((score-value)%50))
	{
		addLife();
		drawLives();
		dripRate=abs(dripRate-100);
		if(!dripRate) dripRate=100;
	}
	return score;
}

Uint16 GAME::play()
{
	MESSAGE pause(screen, scoreFont, "");
	pause.addLine("Paused...");
	drawGameScreen();
	
	int done=0;
	int xMod=0;

	int cost=0;
	Uint32 dripSpeed=50;
	Uint32 nextMove=0;
	Uint32 nextDrip=0;
	Uint32 dropDrips=0;
	Uint32 ticks;
	SDL_Rect src;
	SDL_Rect dest;
	SDL_Rect dripDest;
	SDL_Event event;
	src.x=0;
	src.y=0;
	src.w=16;
	src.h=16;
	
	dest.x=152;
	dest.y=220;
	dest.w=16;
	dest.h=16;
	dripDest.w=16;
	dripDest.h=16;

	
	while (!done)
	{
		/* Check for events */
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					/* if press Ctrl + C, terminate program */
					if ( (event.key.keysym.sym == SDLK_c) && (event.key.keysym.mod & (KMOD_LCTRL | KMOD_RCTRL)) )
						done = 1;
					if(event.key.keysym.sym == SDLK_ESCAPE)
						done=1;
					if(event.key.keysym.sym == SDLK_SPACE)
					{
						if(energy>0)
						{
							speed = 5;
							cost = 1;
						}
					}
					if(event.key.keysym.sym == SDLK_LEFT) xMod=-1;
					if(event.key.keysym.sym == SDLK_RIGHT) xMod=1;
					if(event.key.keysym.sym == SDLK_p )
					{
						pause.displayMessage(130,110);
						drawGameScreen();
					}
					break;
				case SDL_KEYUP:
					if(event.key.keysym.sym == SDLK_SPACE)
					{
						speed = 10;
						cost = 0;
					}
					if((xMod==-1)&&(event.key.keysym.sym == SDLK_LEFT)) xMod=0;
					if((xMod==1)&&(event.key.keysym.sym == SDLK_RIGHT)) xMod=0;					
					break;
				case SDL_JOYBUTTONDOWN:
					/* if press Start button, terminate program */
					if ( event.jbutton.button == VK_START )
						done = 1;
					if(( event.jbutton.button == VK_FA )||( event.jbutton.button == VK_FB )||
					   ( event.jbutton.button == VK_FX )||( event.jbutton.button == VK_FY ))
					{
						if(energy>0)
						{
							speed = 5;
							cost = 1;
						}
					}
					if ( event.jbutton.button == VK_LEFT ) xMod=-1;
					if ( event.jbutton.button == VK_RIGHT ) xMod=1;
					break;
				case SDL_JOYBUTTONUP:
					if(( event.jbutton.button == VK_FA )||( event.jbutton.button == VK_FB )||
					   ( event.jbutton.button == VK_FX )||( event.jbutton.button == VK_FY ))
					{
						speed = 10;
						cost = 0;
					}
					if ((xMod==-1)&&( event.jbutton.button == VK_LEFT )) xMod=0;
					if ((xMod==1)&&( event.jbutton.button == VK_RIGHT )) xMod=0;
					if ( event.jbutton.button == VK_SELECT )
					{
						pause.displayMessage(130,110);
						drawGameScreen();
					}
					break;
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
			}
		}
		ticks=SDL_GetTicks();

		/* updateDrips */
		if(ticks>nextDrip)
		{
			/* add a new drip */
			addDrip();
			nextDrip=SDL_GetTicks()+dripRate;
		}

		/* clear drips */		
		clearDrips();
		 /* clear bucket */
		SDL_FillRect(screen, &dest, SDL_MapRGB(screen->format, 0, 0, 0));
		
		/* if time to move drips then move them */
		if(ticks>dropDrips)
		{
			if(deleteDrips(dest.x))
				done=1;

			modifyDrips();
		dropDrips=SDL_GetTicks()+dripSpeed;
		}
		
		/* If time to move bucket them move it */	
		if(ticks>nextMove)
		{
			dest.x+=xMod;
			if(dest.x<0) dest.x=0;
			if(dest.x>304) dest.x=304;
			energy-=cost;
			if((speed==5) && (energy<1))
			{
				speed=10;
				cost=0;
			}
			nextMove=SDL_GetTicks()+speed;
		}
		
		/* draw the bucket */
		SDL_BlitSurface(bucket, &src, screen, &dest);
		/* draw drips, score and life*/
		drawDrips();
		drawScore();
		drawLives();
		drawEnergy();
		SDL_Flip(screen);
	}
	deleteAllDrips();
	soundBank->playSound(snd_gameOver);
	return score;
}

void GAME::drawGameScreen()
{
	/* blank the screen */
	SDL_Rect rect;
	rect.x=0;
	rect.y=0;
	rect.w=320;
	rect.h=240;
	SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0, 0, 0));	
	
	drawScore();
	
	drawLives();
	
	SDL_Rect Src;
	SDL_Rect Dest;
	
	/* Draw the health and boost icons */
	Src.x=0;
	Src.y=0;
	Src.w=7;
	Src.h=7;

	Dest.x=2;
	Dest.y=0;
	SDL_BlitSurface(healthIcon, &Src, screen, &Dest);
	
	Dest.x=2;
	Dest.y=9;
	SDL_BlitSurface(boostIcon, &Src, screen, &Dest);
		
	SDL_Rect bar;
	bar.x=0;
	bar.y=17;
	bar.w=320;
	bar.h=6;
	SDL_FillRect(screen, &bar, SDL_MapRGB(screen->format, 0, 0, 150));
	bar.x=0;
	bar.y=19;
	bar.w=320;
	bar.h=2;
	SDL_FillRect(screen, &bar, SDL_MapRGB(screen->format, 0, 0, 255));		
	SDL_Flip(screen);
}

int GAME::addLife()
{
	soundBank->playSound(snd_levelUp);
	if(lives<96)
		lives+=16;
	return lives;
}

void GAME::drawLives()
{
	SDL_Rect Src;
	SDL_Rect Dest;
	Src.x = 10;
	Src.y = 0;
	Src.w=96;
	Src.h=7;
	SDL_FillRect(screen, &Src, SDL_MapRGB(screen->format, 0, 0, 0));
	Src.w = lives;
	Dest.y = 0;
	Dest.x=10;
	SDL_BlitSurface(life, &Src, screen, &Dest);
}

void GAME::drawEnergy()
{
	SDL_Rect Src;
	SDL_Rect Dest;
	Src.x = 10;
	Src.y = 9;
	Src.w=96;
	Src.h=7;
	SDL_FillRect(screen, &Src, SDL_MapRGB(screen->format, 0, 0, 0));
	Src.w = energy/100;
	Dest.y = 9;
	Dest.x=10;
	SDL_BlitSurface(life, &Src, screen, &Dest);
}

void GAME::addDrip()
{
	/* add a drip to the end of the list of drips */
	DRIP * d;
	DRIP * list;
	Uint16 x;
	Uint16 y;
	Uint16 s;
	x=rand()%304;
	y=23;
	s=(rand()%3)+1;
	
	list=drips;
	d=new DRIP;
	d->next=NULL;
	d->x=x;
	d->y=y;
	d->speed=s;
	int tmp=rand()%50;
	switch(tmp)
	{
		case 1: d->type=1;
			break;
		case 2:  d->type=2;
			break;
		default: d->type=0;
			break;
	}
	if(list==NULL)
	{
		/* no drips yet create the first entry in the list*/
		drips=d;
	}
	else
	{
		/* There are drips get to the end of the list */
		while(list->next)
		{
			list=list->next;
		}
		list->next=d;
	}
	soundBank->playSound(snd_drop);
}

void GAME::drawDrips()
{
	/* draw each drip from the list of drips */
	SDL_Rect src;
	SDL_Rect dest;
	src.x=0;
	src.y=0;
	src.w=16;
	src.h=16;
	
	DRIP * list;
	list=drips;
	while(list)
	{
		/* draw this drip */
		dest.x=list->x;
		dest.y=list->y;
		SDL_BlitSurface(drip[list->type], &src, screen, &dest);
		list=list->next;
	}
}

void GAME::clearDrips()
{
	/* remove each drip from the list of drips */
	DRIP * list;
	SDL_Rect rect;
	rect.w=16;
	rect.h=16;
	list=drips;
	while(list)
	{
		rect.x=list->x;
		rect.y=list->y;
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0, 0, 0));
		list=list->next;
	}
}

void GAME::modifyDrips()
{
	/* adjust the location of each drip in the list of drips */
	DRIP * list;
	list=drips;
	while(list)
	{
		list->y+=list->speed;
		list=list->next;
	}
}

int GAME::deleteDrips(Uint16 bl)
{
	/* remove all drips from the list which have fallen too far */
	DRIP * list;
	DRIP * prev = NULL;
	DRIP * tmp;
	list=drips;
	int ret=0;
	int del;
	
	while(list)
	{
		del=0;
		if(list->y > 210 )
		{
			/* past the end of the fall */
			/* check if in bucket */
			if(abs(list->x-bl)<threashold)
			{
				soundBank->playSound(snd_caught);
				/* if so add drip speed to score */
				addScore(list->speed);
				/* check to see if a special drip */
				switch(list->type)
				{
					case 1: energy+=5000;
					        if(energy>9600) energy=9600;
						break;
					case 2: energy-=5000;
					        if(energy<0) energy=0;
						break;
				}
			}
			else
			{
				soundBank->playSound(snd_missed);
				/* else deduct drip speed from life */
				lives-=list->speed;
			}
			/* if life <=0 then end game */
			if(lives<=0) ret=1;
			
			/* delete the drip */
			del=1;
		}

		if(del)
		{
			tmp=list;
			list=list->next;
			if(prev)
				prev->next=list;
			else
				drips=list;
			delete tmp;
		}
		else
		{
			prev=list;
			list=list->next;
		}
	}
	return ret;
}

void GAME::deleteAllDrips()
{
	/* adjust the location of each drip in the list of drips */
	DRIP * list;
	DRIP * tmp;
	list=drips;
	while(list)
	{
		tmp=list;
		list=list->next;
		delete tmp;
	}
	drips=NULL;
	/* remove all drips from the list */
}

