#include "fonts.h"
#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>
#include "mapkey.h"
#include "drips.h"
#include "message.h"
#include "soundBank.h"

#ifndef GAMEH
#define GAMEH

#ifdef GP2X
#define GFX_LIFE "pngs/life.png"
#define	GFX_BUCKET "pngs/bucket.png"
#define GFX_DRIP "pngs/drip.png"
#define GFX_GREEN_DRIP "pngs/dripGreen.png"
#define GFX_RED_DRIP "pngs/dripRed.png"
#define GFX_HEALTH_ICON "pngs/healthIcon.png"
#define GFX_BOOST_ICON "pngs/boostIcon.png"
#define SOUND_DRIP_2 "sounds/drip2.wav"
#define SOUND_DRIP_3 "sounds/drip3.wav"
#define SOUND_DRIP_4 "sounds/drip4.wav"
#define SOUND_DRIP_5 "sounds/drip5.wav"
#define SOUND_GAME_OVER "sounds/collapse.wav"
#else
#define GFX_LIFE "pngs\\life.png"
#define	GFX_BUCKET "pngs\\bucket.png"
#define GFX_DRIP "pngs\\drip.png"
#define GFX_GREEN_DRIP "pngs\\dripGreen.png"
#define GFX_RED_DRIP "pngs\\dripRed.png"
#define GFX_HEALTH_ICON "pngs\\healthIcon.png"
#define GFX_BOOST_ICON "pngs\\boostIcon.png"
#define SOUND_DRIP_2 "sounds\\drip2.wav"
#define SOUND_DRIP_3 "sounds\\drip3.wav"
#define SOUND_DRIP_4 "sounds\\drip4.wav"
#define SOUND_DRIP_5 "sounds\\drip5.wav"
#define SOUND_GAME_OVER "sounds\\collapse.wav"

#endif

class GAME
{
	public:
		GAME(SDL_Surface * surface, FONT * font, SOUNDBANK * sounds);
		~GAME();
		Uint16 play();
		void reset();
		
		int sound;
		int music;

	private:
		void drawScore();
		void drawLives();
		void drawEnergy();
		void drawGameScreen();		
		int addScore(int value);
		int addLife();
		void addDrip();
		void drawDrips();
		void clearDrips();
		void modifyDrips();
		int deleteDrips(Uint16 bl);
		void deleteAllDrips();
		
		Uint16 score;
		int lives;
		int energy;
		int oldScore;
		Sint32 speed;
		SDL_Surface * life;
		SDL_Surface * bucket;
		SDL_Surface * drip[3];
		SDL_Surface * screen;
		SDL_Surface * healthIcon;
		SDL_Surface * boostIcon;
		FONT * scoreFont;
		Uint32 dripRate;
		int dripCounter;
		int threashold;
		DRIP * drips;
		SOUNDBANK * soundBank;
		int snd_drop;
		int snd_caught;
		int snd_missed;
		int snd_levelUp;
		int snd_gameOver;
};

#endif
