#include "highScore.h"

HIGHSCORE::HIGHSCORE()
{
}

HIGHSCORE::HIGHSCORE(SDL_Surface * surface, FONT * font, char * t)
{
	strncpy(title, t, HIGHSCORE_TITLE_SIZE);
	title[HIGHSCORE_TITLE_SIZE]='\0';
	maxWidth=strlen(title);
	highScoreFont=font;
	screen=surface;
	int s=50;
	/* Default Entries */
	for(int i=HIGHSCORE_MAX_ENTRIES-1; i>=0; i--)
	{
		names[i][0]='.';
		names[i][1]='.';
		names[i][2]='.';
		scores[i]=s;
		s=s*2;
	}
}

HIGHSCORE::~HIGHSCORE()
{
}

int HIGHSCORE::addScore(Uint16 s)
{
	int i;
	

	if(s<scores[HIGHSCORE_MAX_ENTRIES-1])
		return 0;
	
	for(i=HIGHSCORE_MAX_ENTRIES-2; ((i>=0)&&(s>scores[i])); i--)
	{
		strncpy(names[i+1], names[i], HIGHSCORE_ENTRY_SIZE);
		scores[i+1]=scores[i];
	}
	scores[++i]=s;
	
	getInitials(i, 50,50);
	
	return 1;
}

void HIGHSCORE::getInitials(int i, Uint16 x, Uint16 y)
{
	SDL_Rect rect;
	char initials[4]="...";
	int store[3];
	char letters[]="ABCDEFGHIJKLMNOPQRSTUVWXYZ. ";
	int c=0;
	int p=0;
	rect.x=x;
	rect.y=y;
	rect.w=18+(highScoreFont->getFontWidth()*3);
	rect.h=18+(highScoreFont->getFontWidth());
	drawBorder(screen, rect.x, rect.y, rect.w, rect.h);
	
	x+=8;
	y+=8;
	
	int nextTick=SDL_GetTicks()+FLASH_SPEED;
	SDL_Event event;

	while(c<3)
	{
		/* Check for events */
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					if((event.key.keysym.sym == SDLK_SPACE) || (event.key.keysym.sym == SDLK_RIGHT))
					{
						store[c]=p;
						initials[c++]=letters[p];
					}
					if(event.key.keysym.sym == SDLK_UP)
						++p%=28;
					if(event.key.keysym.sym == SDLK_DOWN)
						if(--p<0) p=27;
					if((event.key.keysym.sym == SDLK_LEFT) && (c>0))
					{
						initials[c--]='.';
						p=store[c];
					}
					break;
					
				case SDL_KEYUP:
					break;

				case SDL_JOYBUTTONDOWN:
					/* if press Start button, terminate program */
					if(( event.jbutton.button == VK_START )||( event.jbutton.button == VK_FA )||
					   ( event.jbutton.button == VK_FB )||( event.jbutton.button == VK_FX )||
						 ( event.jbutton.button == VK_FY )||(event.jbutton.button == VK_RIGHT))
					{
						store[c]=p;
						initials[c++]=letters[p];
					}
					if(event.jbutton.button == VK_UP)
						++p%=28;
					if(event.jbutton.button == VK_DOWN)
						if(--p<0) p=27;
					if((event.jbutton.button == VK_LEFT) && (c>0))
					{
						initials[c--]='.';
						p=store[c];
					}
					break;
				case SDL_JOYBUTTONUP:
					break;
				case SDL_QUIT:
					break;
				default:
					break;
			}
			if(c<0)c=0;
		}
		if(SDL_GetTicks()>nextTick)
		{
			if(initials[c]==' ')
				initials[c]=letters[p];
			else
				initials[c]=' ';
					
			nextTick=SDL_GetTicks()+FLASH_SPEED;
		}
				
		highScoreFont->drawTextOver(screen, x, y, 0,0,0,initials);
		SDL_Flip(screen);
	}
	strncpy(names[i], initials, 3);
}

void HIGHSCORE::displayScores(Uint16 x, Uint16 y)
{
	SDL_Rect rect;
		
	/* Draw the menu border and background */
	rect.x = x;
	rect.y = y;
	int maxWidth;
	maxWidth=strlen(title);
	if(strlen(title)<10)
		maxWidth=10;
	rect.w = 18+(highScoreFont->getFontWidth()*(maxWidth));
	if(strlen(title))
		rect.h = 18+(highScoreFont->getFontHeight()*(HIGHSCORE_MAX_ENTRIES+1));
	else
		rect.h = 18+(highScoreFont->getFontHeight()*(HIGHSCORE_MAX_ENTRIES));
	drawBorder(screen, rect.x, rect.y, rect.w, rect.h);
	
	/* Display the title */
	x+=8;
	y+=8;
	if(strlen(title))
		highScoreFont->drawText(screen, x, y, title);
	
	SDL_Rect src;
	src.x=0;
	src.y=0;
	src.w=16;
	src.h=16;
	
	/* nudge the options down a little. */
	if(strlen(title))
		y+=highScoreFont->getFontHeight();
		
	char line[HIGHSCORE_TITLE_SIZE];
	for(int i=0; i<HIGHSCORE_MAX_ENTRIES; i++)
	{
		sprintf(line,"%c%c%c %06d", names[i][0],names[i][1],names[i][2], scores[i]);
		highScoreFont->drawText(screen, x, y, line);
		y+=highScoreFont->getFontHeight();
	}
	
	SDL_Flip(screen);
	
	waitButtonPress();
}

void HIGHSCORE::loadScores(char * file)
{
	int i;
	FILE * highScores;
	
	if(highScores=fopen(file, "rb"))
	{
		for(i=0;i<HIGHSCORE_MAX_ENTRIES; i++)
		{
			fread(names[i],HIGHSCORE_ENTRY_SIZE, 1, highScores);
			fread(&scores[i], sizeof(scores[i]), 1, highScores);
		}
		fclose(highScores);
	}
}

void HIGHSCORE::saveScores(char * file)
{
	int i;
	FILE * highScores;
	
	if(highScores=fopen(file, "wb"))
	{
		for(i=0;i<HIGHSCORE_MAX_ENTRIES; i++)
		{
			fwrite(names[i],HIGHSCORE_ENTRY_SIZE, 1, highScores);
			fwrite(&scores[i], sizeof(scores[i]), 1, highScores);
		}
		fclose(highScores);
	}
}
