#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_Mixer.h>

#include "fonts.h"
#include "MENU.h"
#include "message.h"
#include "game.h"
#include "mapkey.h"
#include "misc.h"
#include "highScore.h"
#include "soundBank.h"

#define VERSION "0.2"

#ifdef GP2X
#define LARGE_FONT "./fonts/drip_16x16.png"
#define SMALL_FONT "./fonts/drip_8x8.png"
#define SPLASH_SCREEN "./pngs/splashScreen.png"
#define TITLE "./pngs/title.png"
#define BULLET "./pngs/bullet.png"
#define ACTIVE_BULLET "./pngs/active_bullet.png"
#define SOUND_MENU_BLEEP "./sounds/bleep.wav"
#define SOUND_NEW_GAME "./sounds/thunder.wav"
#else
#define LARGE_FONT "fonts\\drip_16x16.png"
#define SMALL_FONT "fonts\\drip_8x8.png"
#define SPLASH_SCREEN "pngs\\splashScreen.png"
#define TITLE "pngs\\title.png"
#define BULLET "pngs\\bullet.png"
#define ACTIVE_BULLET "pngs\\active_bullet.png"
#define SOUND_MENU_BLEEP "sounds\\bleep.wav"
#define SOUND_NEW_GAME "sounds\\thunder.wav"
#endif
/* The screen surface, joystick device */
SDL_Surface *screen = NULL;
SDL_Joystick *joy = NULL;

/* Title Screen */
SDL_Surface * titleScreen = NULL;
SDL_Surface * splashScreen =NULL;

void Terminate(void)
{
	SDL_Quit();
#ifdef GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}

int main (int argc, char *argv[])
{
	int done;
	int menu;
	int menuSound;
	int snd_newGame;
	int args;
	Uint32 screenFlags=SDL_SWSURFACE;
	SDL_Rect src;
	SDL_Rect dest;
	
	// Randomize
	srand(time(NULL));
	src.x=dest.x=0;
	src.y=dest.y=0;
	src.w=320;
	src.h=240;
	
	for(args=1;args<argc;args++)
	{
		if(strncmp(argv[1],"-f",2)==0)
			screenFlags=screenFlags|SDL_FULLSCREEN;
	}
	
	/* Initialize SDL */
	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0) {
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1); 
	}
	atexit (Terminate);

	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16SYS;
	int audio_channels = 2;
	//int audio_buffers = 4096;  Too much delay
	//int audio_buffers = 2048;  Too much delay
	int audio_buffers = 1024;
 
	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) != 0) {
		fprintf(stderr, "Unable to initialize audio: %s\n", Mix_GetError());
		exit(1);
	}

	SDL_ShowCursor(SDL_DISABLE);

	/* Set 320x240 16-bits video mode */
	screen = SDL_SetVideoMode (320, 240, 16, screenFlags);
	if (screen == NULL) {
		fprintf (stderr, "Couldn't set 320x240x16 video mode: %s\n", SDL_GetError ());
		exit (2);
	}
	
	fprintf(stderr, "loading big font\n");
		/* Create Fonts */
	FONT bigFont(LARGE_FONT, 16, 16);
	bigFont.setTransparent(255, 0, 255, SDL_ALPHA_TRANSPARENT);

	fprintf(stderr, "loading small font\n");	
	FONT smallFont(SMALL_FONT, 8, 8);
	smallFont.setTransparent(255, 0, 255, SDL_ALPHA_TRANSPARENT);
	
	/* create the sound bank */
	SOUNDBANK soundBank;
	fprintf(stderr, "loading sound 1\n");
	menuSound=soundBank.loadSound(SOUND_MENU_BLEEP);
	fprintf(stderr, "loading sound 2\n");
	snd_newGame=soundBank.loadSound(SOUND_NEW_GAME);
	
	fprintf(stderr, "create game object\n");
	GAME game(screen, &bigFont, &soundBank);

	fprintf(stderr, "create menu object\n");
	MENU mainMenu(screen, &bigFont, "", BULLET, ACTIVE_BULLET);
	
	fprintf(stderr, "create message object\n");
	MESSAGE credits(screen, &smallFont, "Credits");  

	fprintf(stderr, "create highscore object\n");
	HIGHSCORE highScoreTable(screen, &bigFont, "High Scores");

	fprintf(stderr, "loading highscores\n");
	highScoreTable.loadScores("drip.hst");
	/* Check and open joystick device */
	if (SDL_NumJoysticks() > 0) {
		joy = SDL_JoystickOpen(0);
		if(!joy) {
			fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		}
	}

	/* display the splash screen */
	fprintf(stderr, "loading splash screen\n");
	splashScreen=IMG_Load(SPLASH_SCREEN);
	if(!splashScreen)
		fprintf(stderr, "IMG_Load: %s\n", IMG_GetError());
	fprintf(stderr, "displaying splash screen\n");
	SDL_BlitSurface(splashScreen, &src, screen, &dest);
	SDL_Flip(screen);
	waitButtonPress();
	

	/* Load the title screen */
	titleScreen=IMG_Load(TITLE);

  /* Display main menu */
	mainMenu.addEntry("Play");
	mainMenu.addEntry("Sound Effects On");

	mainMenu.addEntry("High Score Table");		
	mainMenu.addEntry("Credits");
	mainMenu.addEntry("Quit");

	credits.addLine(" Coding by Jason Cooper");
	credits.addLine(" Graphics by Jason Cooper");
	credits.addLine(" Sound Effects by Jason Cooper");
	credits.addLine(" Fonts by Jason Cooper");
	
	done=0;
	menu=0;
	int menuChoice;
	while(!done)
	{
		SDL_BlitSurface(titleScreen, &src, screen, &dest);
		smallFont.drawText(screen, 290,230, VERSION);
		SDL_Flip(screen);
		if(!menu)
			waitButtonPress();
		if(soundBank.soundMuted())
			mainMenu.setEntry(2,"Sound Effects Off");
		else
			mainMenu.setEntry(2,"Sound Effects On");
			
		menuChoice=mainMenu.displayMenu(15,120);
		soundBank.playSound(menuSound);
			
		switch(menuChoice)
		{
			case 1:	
							soundBank.playSound(snd_newGame);
							game.reset();
			        highScoreTable.addScore(game.play());
			        highScoreTable.displayScores(10,10);
			        menu=0;
				break;
			case 2:	if(soundBank.soundMuted())
								soundBank.unmuteSound();
							else
								soundBank.muteSound();
							menu=1;
				break;
			case 3: highScoreTable.displayScores(10,10);
							menu=1;
				break;
			case 4: credits.displayMessage(30,30);
							menu=1;
				break;
			case 5: done=1;
							menu=0;
				break;
		}
	}
	
	/* Clean up title screen */
	if(titleScreen)
		SDL_FreeSurface(titleScreen);
	
	if(splashScreen)
		SDL_FreeSurface(splashScreen);
		
	highScoreTable.saveScores("drip.hst");
	return 0;
}
